@@ -226,8 +226,6 @@ class LaharFlowData(object):
                 self._get_parameters_from_runinfo()
             self._get_max_depth()
             self._get_max_speed()
-
-            self.rasters=dict()
     
     def _get_parameters_from_inputfile(self):
         if self.zipped:
@@ -491,7 +489,7 @@ class LaharFlowData(object):
         except:
             return None
     
-    def raster(self, filename, var=None, full_domain=False, nodata=np.nan, masked=True, crs=None, store_local=True):
+    def raster(self, filename, var=None, full_domain=False, nodata=np.nan, masked=True, crs=None):
         filetype = self._file_type(filename)
         
         filename = self._valid_input_file(filename)
@@ -546,7 +544,7 @@ class LaharFlowData(object):
             ds[col].attrs["units"] = fmt['units']
             ds[col].attrs["symbol"] = fmt['symbol']
         
-        if filetype=='snapshot' and 'concentration' in data_cols:
+        if filetype=='snapshot' and 'concentration' not in data_cols:
             ds['concentration'] = ds['mass_of_solids']
             ds['concentration'] = np.divide(ds['mass_of_solids'], ds['mass_per_unit_area'], where=ds['mass_per_unit_area'].where(ds['mass_per_unit_area'].data>1e-8))
             fmt = _get_LF_output_fmt('concentration')
@@ -562,9 +560,6 @@ class LaharFlowData(object):
 
         if crs is not None:
             ds = ds.rio.reproject(crs)
-        
-        if store_local:
-            self.rasters[os.path.splitext(filename)] = ds
 
         return ds
 
@@ -633,14 +628,13 @@ class LaharFlowData(object):
 
         return {'MaxErosion': MaxErosion, 'MaxDeposit': MaxDeposit}
     
-    def max_bed_evolution(self, vmin=None, vmax=None):
+    def max_bed_evolution(self, data=None, vmin=None, vmax=None):
         last_outfile = self.get_snapshot_files()[-1]
-        if os.path.splitext(last_outfile) not in self.rasters.keys():
-            data = self.raster(last_outfile)
-        else:
+        
+        if data is None:
             data = self.rasters[last_outfile]
-            if 'elevation_change' not in list(data.keys()):
-                data = self.raster(last_outfile)
+        if 'elevation_change' not in list(data.keys()):
+            data = self.raster(last_outfile)
             
         bt = data['elevation_change']
         
@@ -658,15 +652,13 @@ class LaharFlowData(object):
         
         self.MaxDepth = np.amax(h)
     
-    def max_depth(self, vmin=None):
+    def max_depth(self, data=None, vmin=None):
 
-        if 'MaxHeights' not in self.rasters.keys():
-            data = self.raster('MaxHeights.txt')
-        else:
+        if data is None:
             data = self.rasters['MaxHeights']
-            if 'maximum_depth' not in list(data.keys()):
-                data = self.raster('MaxHeights.txt')
-        data = self.raster('MaxHeights.txt')
+        if 'maximum_depth' not in list(data.keys()):
+            data = self.raster('MaxHeights.txt')
+        
         maxh = data['maximum_depth']
 
         if vmin is not None:
@@ -680,9 +672,12 @@ class LaharFlowData(object):
         
         self.MaxSpeed = np.amax(spd)
         
-    def max_speed(self, vmin=None):
+    def max_speed(self, data=None, vmin=None):
 
-        data = self.raster('MaxSpeeds.txt')
+        if data is None:
+            data = self.raster('MaxSpeeds.txt')
+        if 'maximum_speed' not in list(data.keys()):
+            data = self.raster('MaxSpeeds.txt')
         maxspd = data['maximum_speed']
 
         if vmin is not None:
@@ -690,9 +685,12 @@ class LaharFlowData(object):
 
         return maxspd
     
-    def inundation_time(self, vmin=None):
+    def inundation_time(self, data=None, vmin=None):
 
-        data = self.raster('InundationTime.txt')
+        if data is None:
+            data = self.raster('InundationTime.txt')
+        if 'inundation_time' not in list(data.keys()):
+            data = self.raster('InundationTime.txt')
         itime = data['inundation_time']
 
         if vmin is not None:
@@ -801,9 +799,9 @@ class LaharFlowData(object):
         
         return g
 
-    def max_depth_contours(self, name='MaxHeights', cntrs=0.1, vmin=0.1):
+    def max_depth_contours(self, data=None, name='MaxHeights', cntrs=0.1, vmin=0.1):
         
-        maxh = self.max_depth(vmin=vmin)
+        maxh = self.max_depth(data=data, vmin=vmin)
 
         if maxh is not None:
             #maxh = maxh['maximum_depth']
@@ -824,9 +822,9 @@ class LaharFlowData(object):
         else:
             return None
     
-    def max_speed_contours(self,name='MaxSpeeds',cntrs=1, vmin=None):
+    def max_speed_contours(self, data=None, name='MaxSpeeds',cntrs=1, vmin=None):
         
-        maxspd = self.max_speed(vmin=vmin)
+        maxspd = self.max_speed(data=data, vmin=vmin)
 
         if maxspd is not None:
             maxspd = maxspd['maximum_speed']
@@ -847,9 +845,9 @@ class LaharFlowData(object):
         else:
             return None
 
-    def inundation_time_contours(self,name='InundationTime',cntrs=0,vmin=0):
+    def inundation_time_contours(self, data=None, name='InundationTime',cntrs=0,vmin=0):
         
-        itime = self.inundation_time(vmin=vmin)
+        itime = self.inundation_time(data=data, vmin=vmin)
 
         if itime is not None:
             if type(cntrs)==np.ndarray:
@@ -870,15 +868,57 @@ class LaharFlowData(object):
         else:
             return None
     
-    def POI_value(self, filename, x, y, var=None, full_domain=False, nodata=np.nan, fill_value=0, interp_method='linear'):
+    def POI_value(self,  x, y, data=None, filename=None, var=None, full_domain=False, nodata=np.nan, fill_value=0, interp_method='linear'):
         
-        if var is not None:
+        if data is None and filename is None:
+            raise RuntimeError('cannot have both data=None and filename=None')
+        
+        if data is not None and filename is not None:
+            raise RuntimeError('cannot give both data and filename')
+
+        if not isinstance(var, list):
             var = [var]
 
         if interp_method not in ['nearest', 'linear'] and fill_value is None:
             RuntimeError("fill_value must be specified unless using 'nearest' or 'linear' interp_method.")
 
-        data = self.raster(filename, var=var, full_domain=full_domain, nodata=np.nan, masked=True, crs=None)
+        if filename is not None:
+            data = self.raster(filename, var=var, full_domain=full_domain, nodata=np.nan, masked=True, crs=None)
+
+        if fill_value is not None:
+            data = data.fillna(fill_value)
+        
+        value = data.interp(x=x, y=y, method=interp_method)
+
+        val = dict()
+        vars = var if var is not None else list(data.keys())
+        for v in vars:
+            val[v] = value[v].values
+        
+        return val
+    
+    def transect_value(self, x, y, filename=None, data=None, var=None, full_domain=False, nodata=np.nan, fill_value=0, interp_method='linear'):
+        
+        if data is None and filename is None:
+            raise RuntimeError('cannot have both data=None and filename=None')
+        
+        if data is not None and filename is not None:
+            raise RuntimeError('cannot give both data and filename')
+
+        if not isinstance(var, list):
+            var = [var]
+
+        xA = xa.DataArray(x, dims="z")
+        yA = xa.DataArray(y, dims="z")
+
+        if interp_method not in ['nearest', 'linear']:
+            raise RuntimeError("currently only works with nearest or linear interpolation")
+
+        if interp_method not in ['nearest', 'linear'] and fill_value is None:
+            raise RuntimeError("fill_value must be specified unless using 'nearest' or 'linear' interp_method.")
+
+        if filename is not None:
+            data = self.raster(filename, var=var, full_domain=full_domain, nodata=np.nan, masked=True, crs=None)
 
         if fill_value is not None:
             data = data.fillna(fill_value)
@@ -889,28 +929,8 @@ class LaharFlowData(object):
         vars = var if var is not None else list(data.keys())
         for v in vars:
             val[v] = value[v].values
-        
+
         return val
-    
-    def transect_value(self, filename, x, y, var=None, full_domain=False, nodata=np.nan, fill_value=0, interp_method='linear'):
-        
-        if var is not None:
-            var = [var]
-
-        xA = xa.DataArray(x, dims="z")
-        yA = xa.DataArray(y, dims="z")
-
-        if interp_method not in ['nearest', 'linear'] and fill_value is None:
-            RuntimeError("fill_value must be specified unless using 'nearest' or 'linear' interp_method.")
-
-        data = self.raster(filename, var=var, full_domain=full_domain, nodata=np.nan, masked=True, crs=None)
-
-        if fill_value is not None:
-            data = data.fillna(fill_value)
-        
-        value = data.interp(x=xA, y=yA, method=interp_method)
-
-        return value
 
     def save_gtiff(self, fileIn, fileOut=None, crs=None, **kwargs):
 
